package FSA;

import java.io.*;

public class DFA
{
    private Transition tA [];
    private String initString;
    private int initWeight;
    private char type;

    /* constructors are overloaded for various types of
     * automata
     */
    public DFA( Transition inittA [], String i, int w) {
	tA=inittA;
	initString=i;
	initWeight=w;
	type='p';
    }

    public DFA( Transition inittA [], String i) {
	this(inittA,i,0);
	type='t';
    }

    public DFA( Transition inittA [], int w) {
	this(inittA,"",w);
	type='w';
    }

    public DFA( Transition inittA []) {
	this(inittA,"",0);
	type='r';
    }

    /* simple boolean methods to get the type of the automaton */
    public boolean Recognizer() {
	return type=='r';
    }

    public boolean Transducer() {
	return type=='t';
    }

    public boolean WeightedRecognizer() {
	return type=='w';
    }

    public boolean WeightedTransducer() {
	return type=='p';
    }

    /* apply the automaton as a UNIX filter, i.e. reading lines
     * from standard input, and apply the automaton to each line
     */
    public void filter ()
    {
	BufferedReader in = new BufferedReader( new 
	                     InputStreamReader ( System.in ));
	String oneLine, out;
	Integer outw;
	
	try {	
	    if (Recognizer())
		while( (oneLine = in.readLine()) != null ) 
		    {	if (accepts(oneLine)) 
			    System.err.println("yes");
			else
			    System.err.println("no");
		    }
	    else if (Transducer())
		while( (oneLine = in.readLine()) != null ) 
		    {	out=transduces(oneLine);
			if (out != null) 
			    System.out.println(out);
			else
			    System.err.println("no");
		    }
	    else if (WeightedRecognizer())
		while( (oneLine = in.readLine()) != null ) 
		    {   outw=weighs(oneLine);
			if (outw != null) 
			    System.out.println(outw);
			else
			    System.err.println("no");
		    }
	}
	catch( Exception e )
	    {System.err.println(e);}
    }

    /* returns true if the string is recognized by the automaton;
     * returns false otherwise
     */
    public boolean accepts( String in )
    {
	int line=1;
	int i=0;
	char c;

	while (i < in.length()) {
	    c=in.charAt(i);
	    line=tA[line].next;
	    while (!tA[line].last && c > tA[line].symbol) {  
		line++;
	    }
	    if (c != tA[line].symbol) 
		line=0;
	    i++;
	}
	return tA[line].fin;
    }

    /* returns the transduction of the input string, or `null' if
     * the input is not in the domain of the transducer
     */
    public String transduces(String in)
    {
	int line=1;
	int i=0;
	char c;
	String out=initString;

	while (i < in.length()) {
	    c=in.charAt(i);
	    line=tA[line].next;
	    while (!tA[line].last && c > tA[line].symbol) {  
		line++;
	    }
	    if (c != tA[line].symbol) 
		line=0;
	    out = out+tA[line].out_string;
	    i++;
	}
	out= out+tA[line].final_out_string;
	if (tA[line].fin)
	    return out;
	else
	    return null;
    }

    /* returns the weight of the input string, or `null' if the input
     * is not in the domain of the transducer 
     */
    public Integer weighs( String in  )
    {
	int line=1;
	int i=0;
	char c;

	int out=initWeight;

	while (i < in.length()) {
	    c=in.charAt(i);
	    line=tA[line].next;
	    while (!tA[line].last && c > tA[line].symbol) {  
		line++;
	    }
	    if (c != tA[line].symbol) 
		line=0;
	    out = out+tA[line].out_weight;
	    i++;
	}
	out= out+tA[line].final_out_weight;
	if (tA[line].fin)
	    return new Integer(out);
	else 
	    return null;
    }

    /* Usage: java <file> -w
     *        java <file>
     * In the first form, runs the gui() method of the automaton in
     * <file>; in the second form, runs the filter() method of the
     * automaton in <file>
     */
    public static void main(String argv[], DFAapplet aut) 
    {
	if (argv.length == 1 && argv[0].equals("-w"))
	    aut.gui();
	else if (argv.length == 0)
	    aut.automaton().filter();
	else 
	    {
		System.err.println("Usage: java <file> -w (with gui)");
		System.err.println("       java <file>    (without gui)");
	    }
    }
}
    
